#!/usr/bin/env python

# 
# Text layer (existing) to individual character layers
# Based on character-layers-from-text.py created by Tin Tran

#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Tin Tran: Initial release.
# Rel 2: Tin Tran: Put every layers inside a layer group (requested by Blighty) and creates a new image for friendliness.
# Rel 3: MrQ: create layer group as an option, added color selection, added "Layers to image size?" 

import math
import string
from gimpfu import *
from array import array
import sys
import random
global textlayer 
textlayers  = []

def python_fu_layer_to_new_image(image, layer):

	active_layer = pdb.gimp_image_get_active_layer(image)
	w = pdb.gimp_drawable_width(active_layer)
	h = pdb.gimp_drawable_height(active_layer)
	
	new_image = pdb.gimp_image_new(w, h, 0)
	display = pdb.gimp_display_new(new_image)
	active = pdb.gimp_layer_new_from_drawable(layer, new_image)
	pdb.gimp_image_insert_layer(new_image, active, None, 0)
	pdb.gimp_image_resize_to_layers(new_image)	# when the source image has layers of different dimensions
	active_new = pdb.gimp_image_get_active_layer(new_image)	
	active_new.name = active_layer.name
########## remove the extension from the layer name ############	
	sourcename = active_layer.name
	if '.' in sourcename:
		split = sourcename.split(".", 1)
		sourcename = split[0]
		active_new.name = sourcename
################################################################
	pdb.gimp_displays_flush()
	
def python_character_layers_from_text_existing(image, textlayer,color, justify,resize, group,lg_name):

	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()

	gimp.set_foreground(color)	
	active_layer = pdb.gimp_image_get_active_layer(image)
	copy=python_fu_layer_to_new_image(image, active_layer)
	text = pdb.gimp_text_layer_get_text(textlayer)
	
	font = pdb.gimp_text_layer_get_font(textlayer)
	font_size,unit = pdb.gimp_text_layer_get_font_size(textlayer)
	#pdb.gimp_image_resize_to_layers(image)
	new_image = pdb.gimp_image_new(image.width,image.height,RGB)
	new_display = pdb.gimp_display_new(new_image) #creates a display for our new image
	new_layer = pdb.gimp_layer_new_from_drawable(active_layer, new_image)
	pdb.gimp_image_insert_layer(new_image,new_layer,None,0)
	pdb.gimp_item_set_name(new_layer,"Character Layers from Text")

	offset_x,offset_y = pdb.gimp_drawable_offsets(textlayer)
	count = 0
	lines = text.split("\n")
	starting_y = offset_y
	for line in lines:
		textlayers = []
		totalwidth = 0
		for i in range(0,len(line)):
			char = line[i]
			if ((char == " ") and (i > 0) and (i < len(line)-1)): #if space and it's between 2 chararacters we treat it special
				before = line[i-1]
				space = line[i]
				after = line[i+1]
				special = before + space + after
				special_layer = pdb.gimp_text_fontname(new_image,None,0,0,special,0,TRUE,font_size,unit,font)
				before_layer = pdb.gimp_text_fontname(new_image,None,0,0,before,0,TRUE,font_size,unit,font)
				after_layer = pdb.gimp_text_fontname(new_image,None,0,0,after,0,TRUE,font_size,unit,font)
				layer_height = special_layer.height
				layer_width = special_layer.width - before_layer.width - after_layer.width
				text_layer = pdb.gimp_layer_new(new_image,layer_width,layer_height,RGBA_IMAGE,"Space",100,LAYER_MODE_NORMAL)
				pdb.gimp_image_insert_layer(new_image,text_layer,None,0)
				pdb.gimp_image_remove_layer(new_image,before_layer)
				pdb.gimp_image_remove_layer(new_image,after_layer)
				pdb.gimp_image_remove_layer(new_image,special_layer)
			else:
				text_layer = pdb.gimp_text_fontname(new_image,None,0,0,char,0,TRUE,font_size,unit,font)

			totalwidth += text_layer.width #increment total width to calculate for placement and justify.
			textlayers.append(text_layer)
		if (justify == 0): #left justify
			starting_x = offset_x
		elif (justify == 1): #centered
			starting_x = offset_x + (textlayer.width/2) - totalwidth/2
		elif (justify == 2): #right justify
			starting_x = offset_x + textlayer.width - totalwidth
		for text_layer in textlayers: #loop through and move it into its placement
			pdb.gimp_layer_set_offsets(text_layer,starting_x,starting_y)
			starting_x += text_layer.width
		starting_y += text_layer.height #increment it for next round/line

	pdb.gimp_image_remove_layer(new_image,new_layer)
	
	if resize==1:
		for layer in new_image.layers:

			pdb.gimp_image_resize_to_layers(new_image)
			pdb.gimp_layer_resize_to_image_size(layer)			
	if group==1:
	
		group_new = pdb.gimp_layer_group_new(new_image)
		parent = pdb.gimp_item_get_parent(group_new)
		position = pdb.gimp_image_get_item_position(new_image, text_layer)
		pdb.gimp_image_insert_layer(new_image,group_new, None, 0) 
		group_new.name = str(lg_name)
		applylayers = []
		for applylayer in new_image.layers[1:]:
			pdb.gimp_image_reorder_item(new_image, applylayer, group_new, position)
			pdb.gimp_image_lower_item_to_bottom(new_image, applylayer)

	pdb.gimp_image_resize_to_layers(new_image)
	
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()


register(
	"python_character_layers_from_text_existing",
	"Text layer (existing) to new image & individual character layers.\nIf you use a single line of text, you must end it with a space. Ver.3.",
	"Character layers from Text",
	"Tin Tran/MrQ",
	"GPL 3",
	"2019/23",
	"<Image>/Filters/Text/Character Layers from Text...",             #Menu path
	"RGB*, GRAY*",
	[
	(PF_COLOR,  "color",   "Character layers Color: ", (0,0,0)),	
	(PF_OPTION, "justify",   "Justified:", 0, ["Left Justified","Centered","Right Justified"]),
	(PF_TOGGLE, "resize",  "Layers to Image Size?", 0),	
	(PF_TOGGLE, "group",   "Create New Layers Group:  \n(don't use  for playback\n or export as animation)", 0),
	(PF_STRING, "lg_name", " Name group", "Character 1"),	
	],
	[],
	python_character_layers_from_text_existing)


main()
